#!/bin/sh

# PowerVR SGX DDK for Embedded Linux - installation script
#
# Copyright	2004-2006 by Imagination Technologies Limited.
#				All rights reserved.  No part of this software, either
#				material or conceptual may be copied or distributed,
#				transmitted, transcribed, stored in a retrieval system
#				or translated into any human or computer language in any
#				form by any means, electronic, mechanical, manual or
#				other-wise, or disclosed to third parties without the
#				express written permission of Imagination Technologies
#				Limited, Unit 8, HomePark Industrial Estate,
#				King's Langley, Hertfordshire, WD4 8LZ, U.K.

# Auto-generated for intel_linux from 
#	$RCSfile: common.m4 $ $Revision: 1.9 $
#	$RCSfile: install.sh.m4 $ $Revision: 1.139 $
#	$RCSfile: install.sh.m4 $ $Revision: 1.6 $
#

# PVR Consumer services version number
#
PVRVERSION=1.7.862890

# Where we record what we did so we can undo it.
#
DDK_INSTALL_LOG=/etc/powervr_ddk_install.log

DISCIMAGE=/

# basic installation function
# $1=blurb
#
bail()
{
    echo "$1" >&2
    echo "" >&2
    echo "Installation failed" >&2
    exit 1
}

# basic installation function
# $1=fromfile, $2=destfilename, $3=blurb, $4=chmod-flags, $5=chown-flags
#
install_file()
{
	DESTFILE=${DISCIMAGE}$2
	DESTDIR=`dirname $DESTFILE`

	if [ ! -e $1 ]; then
		 	[ -n "$VERBOSE" ] && echo "skipping file $1 -> $2"
		 return
	fi
	
	# Destination directory - make sure it's there and writable
	#
	if [ -d "${DESTDIR}" ]; then
		if [ ! -w "${DESTDIR}" ]; then
			bail "${DESTDIR} is not writable."
		fi
	else
		$DOIT mkdir -p ${DESTDIR} || bail "Couldn't mkdir -p ${DESTDIR}"
		[ -n "$VERBOSE" ] && echo "Created directory `dirname $2`"
	fi

	# Delete the original so that permissions don't persist.
	#
	$DOIT rm -f $DESTFILE

	$DOIT cp -f $1 $DESTFILE || bail "Couldn't copy $1 to $DESTFILE"
	$DOIT chmod $4 ${DESTFILE}
	$DOIT chown $5 ${DESTFILE}

	echo "$3 `basename $1` -> $2"
	$DOIT echo "file $2" >>${DISCIMAGE}${DDK_INSTALL_LOG}
}

# Install a symbolic link
# $1=fromfile, $2=destfilename
#
install_link()
{
	DESTFILE=${DISCIMAGE}$2
	DESTDIR=`dirname $DESTFILE`

	if [ ! -e ${DESTDIR}/$1 ]; then
		 [ -n "$VERBOSE" ] && echo $DOIT "skipping link ${DESTDIR}/$1"
		 return
	fi

	# Destination directory - make sure it's there and writable
	#
	if [ -d "${DESTDIR}" ]; then
		if [ ! -w "${DESTDIR}" ]; then
			bail "${DESTDIR} is not writable."
		fi
	else
		$DOIT mkdir -p ${DESTDIR} || bail "Couldn't mkdir -p ${DESTDIR}"
		[ -n "$VERBOSE" ] && echo "Created directory `dirname $2`"
	fi

	# Delete the original so that permissions don't persist.
	#
	$DOIT rm -f $DESTFILE

	$DOIT ln -s $1 $DESTFILE || bail "Couldn't link $1 to $DESTFILE"
	$DOIT echo "link $2" >>${DISCIMAGE}${DDK_INSTALL_LOG}
	[ -n "$VERBOSE" ] && echo " linked `basename $1` -> $2"
}

# Tree-based installation function
# $1 = fromdir $2=destdir $3=blurb
#
install_tree()
{
	# Make the destination directory if it's not there
	#
	if [ ! -d ${DISCIMAGE}$2 ]; then
		$DOIT mkdir -p ${DISCIMAGE}$2 || bail "Couldn't mkdir -p ${DISCIMAGE}$2"
	fi
	if [ "$DONTDOIT" ]; then
		echo "### tar -C $1 -cf - . | tar -C ${DISCIMAGE}$2 -x${VERBOSE}f -" 
	else
		tar -C $1 -cf - . | tar -C ${DISCIMAGE}$2 -x${VERBOSE}f -
	fi
	if [ $? = 0 ]; then
		echo "Installed $3 in ${DISCIMAGE}$2"
		$DOIT echo "tree $2" >>${DISCIMAGE}${DDK_INSTALL_LOG}
	else
		echo "Failed copying $3 from $1 to ${DISCIMAGE}$2"
	fi
}

# Uninstall something.
#
uninstall()
{
	if [ ! -f ${DISCIMAGE}${DDK_INSTALL_LOG} ]; then
		echo "Nothing to un-install."
		return;
	fi

	BAD=0
	VERSION=""
	while read type data; do
		case $type in
		version)	# do nothing
			echo "Uninstalling existing version $data"
			VERSION="$data"
			;;
		link|file) 
			if [ -z "$VERSION" ]; then
				BAD=1;
				echo "No version record at head of ${DISCIMAGE}${DDK_INSTALL_LOG}"
			elif ! $DOIT rm -f ${DISCIMAGE}${data}; then
				BAD=1;
			else
				[ -n "$VERBOSE" ] && echo "Deleted $type $data"
			fi
			;;
		tree)		# so far, do nothing
			;;
		esac
	done < ${DISCIMAGE}${DDK_INSTALL_LOG};

	if [ $BAD = 0 ]; then
		echo "Uninstallation completed."
		$DOIT rm -f ${DISCIMAGE}${DDK_INSTALL_LOG}
	else
		echo "Uninstallation failed!!!"
	fi
}

# Help on how to invoke
#
usage()
{
	echo "usage: $0 [options...]"
	echo ""
	echo "Options: -v            verbose mode"
	echo "         -n            dry-run mode"
	echo "         -u            uninstall-only mode"
	echo "         --no-pvr      don't install PowerVR driver components"
	echo "         --no-x        don't install X window system"
	echo "         --no-display  don't install integrated PowerVR display module"
	echo "         --no-bcdevice don't install buffer class device module"
	echo "         --root path   use path as the root of the install file system"
	exit 1
}

install_pvr()
{
	$DOIT echo "version $PVRVERSION" >${DISCIMAGE}${DDK_INSTALL_LOG}

	# Install the standard libraries
	#
	install_file libGLES_CM.so /usr/lib/libGLES_CM.so.$PVRVERSION "shared library" 0644 0:0
	install_link libGLES_CM.so.$PVRVERSION /usr/lib/libGLES_CM.so
	install_link libGLES_CM.so.$PVRVERSION /usr/lib/libGLESv1_CM.so.1


	install_file libusc.so /usr/lib/libusc.so.$PVRVERSION "shared library" 0644 0:0
	install_link libusc.so.$PVRVERSION /usr/lib/libusc.so

	install_file libGLESv2.so /usr/lib/libGLESv2.so.$PVRVERSION "shared library" 0644 0:0
	install_link libGLESv2.so.$PVRVERSION /usr/lib/libGLESv2.so
	install_link libGLESv2.so.$PVRVERSION /usr/lib/libGLESv2.so.2

	install_file libglslcompiler.so /usr/lib/libglslcompiler.so.$PVRVERSION "shared library" 0644 0:0
	install_link libglslcompiler.so.$PVRVERSION /usr/lib/libglslcompiler.so

	install_file libOpenVG.so /usr/lib/libOpenVG.so.$PVRVERSION "shared library" 0644 0:0
	install_link libOpenVG.so.$PVRVERSION /usr/lib/libOpenVG.so
	install_file libOpenVGU.so /usr/lib/libOpenVGU.so.$PVRVERSION "shared library" 0644 0:0
	install_link libOpenVGU.so.$PVRVERSION /usr/lib/libOpenVGU.so


	install_file libIMGegl.so /usr/lib/libIMGegl.so.$PVRVERSION "shared library" 0644 0:0
	install_link libIMGegl.so.$PVRVERSION /usr/lib/libIMGegl.so
	install_file libEGL.so /usr/lib/libEGL.so.$PVRVERSION "shared library" 0644 0:0
	install_link libEGL.so.$PVRVERSION /usr/lib/libEGL.so
	install_link libEGL.so.$PVRVERSION /usr/lib/libEGL.so.1

	install_file libpvr2d.so /usr/lib/libpvr2d.so.$PVRVERSION "shared library" 0644 0:0
	install_link libpvr2d.so.$PVRVERSION /usr/lib/libpvr2d.so

	install_file libpvrPVR2D_BLITWSEGL.so /usr/lib/libpvrPVR2D_BLITWSEGL.so.$PVRVERSION "shared library" 0644 0:0
	install_link libpvrPVR2D_BLITWSEGL.so.$PVRVERSION /usr/lib/libpvrPVR2D_BLITWSEGL.so
	install_file libpvrPVR2D_FLIPWSEGL.so /usr/lib/libpvrPVR2D_FLIPWSEGL.so.$PVRVERSION "shared library" 0644 0:0
	install_link libpvrPVR2D_FLIPWSEGL.so.$PVRVERSION /usr/lib/libpvrPVR2D_FLIPWSEGL.so
	install_file libpvrPVR2D_FRONTWSEGL.so /usr/lib/libpvrPVR2D_FRONTWSEGL.so.$PVRVERSION "shared library" 0644 0:0
	install_link libpvrPVR2D_FRONTWSEGL.so.$PVRVERSION /usr/lib/libpvrPVR2D_FRONTWSEGL.so
	install_file libpvrPVR2D_LINUXFBWSEGL.so /usr/lib/libpvrPVR2D_LINUXFBWSEGL.so.$PVRVERSION "shared library" 0644 0:0
	install_link libpvrPVR2D_LINUXFBWSEGL.so.$PVRVERSION /usr/lib/libpvrPVR2D_LINUXFBWSEGL.so

	install_file libpvrPVR2D_DRIWSEGL.so /usr/lib/libpvrPVR2D_DRIWSEGL.so.$PVRVERSION "shared library" 0644 0:0
	install_link libpvrPVR2D_DRIWSEGL.so.$PVRVERSION /usr/lib/libpvrPVR2D_DRIWSEGL.so

	install_file libsrv_um.so /usr/lib/libsrv_um.so.$PVRVERSION "shared library" 0644 0:0
	install_link libsrv_um.so.$PVRVERSION /usr/lib/libsrv_um.so
	install_file libsrv_init.so /usr/lib/libsrv_init.so.$PVRVERSION "shared library" 0644 0:0
	install_link libsrv_init.so.$PVRVERSION /usr/lib/libsrv_init.so
	install_file libPVRScopeServices.so /usr/lib/libPVRScopeServices.so.$PVRVERSION "shared library" 0644 0:0
	install_link libPVRScopeServices.so.$PVRVERSION /usr/lib/libPVRScopeServices.so


	install_file libegl4ogl.so /usr/lib/libegl4ogl.so.$PVRVERSION "shared library" 0644 0:0
	install_link libegl4ogl.so.$PVRVERSION /usr/lib/libegl4ogl.so

	install_file libPVROGL_MESA.so /usr/lib/libPVROGL_MESA.so.$PVRVERSION "shared library" 0644 0:0
	install_link libPVROGL_MESA.so.$PVRVERSION /usr/lib/libPVROGL_MESA.so

	install_file pvr_dri.so /usr/lib/dri/pvr_dri.so "PVR GL DRI module" 0755 0:0

	# Install the standard executables
	#

	install_file pvrsrvinit /usr/local/bin/pvrsrvinit "binary" 0755 0:0
	install_file sgx_init_test /usr/local/bin/sgx_init_test "binary" 0755 0:0



	# Install the standard unittests
	#


	install_file services_test /usr/local/bin/services_test "binary" 0755 0:0
	install_file sgx_blit_test /usr/local/bin/sgx_blit_test "binary" 0755 0:0
	install_file sgx_clipblit_test /usr/local/bin/sgx_clipblit_test "binary" 0755 0:0
	install_file sgx_flip_test /usr/local/bin/sgx_flip_test "binary" 0755 0:0
	install_file sgx_render_flip_test /usr/local/bin/sgx_render_flip_test "binary" 0755 0:0
	install_file pvr2d_test /usr/local/bin/pvr2d_test "binary" 0755 0:0



#	install_file gles1test1 /usr/local/bin/gles1test1 "binary" 0755 0:0
#	install_file gles1_texture_stream /usr/local/bin/gles1_texture_stream "binary" 0755 0:0

#	install_file gles2test1 /usr/local/bin/gles2test1 "binary" 0755 0:0
	install_file glsltest1_vertshader.txt /usr/local/bin/glsltest1_vertshader.txt "shader" 0644 0:0
	install_file glsltest1_fragshaderA.txt /usr/local/bin/glsltest1_fragshaderA.txt "shader" 0644 0:0
	install_file glsltest1_fragshaderB.txt /usr/local/bin/glsltest1_fragshaderB.txt "shader" 0644 0:0
#	install_file gles2_texture_stream /usr/local/bin/gles2_texture_stream "binary" 0755 0:0

#	install_file ovg_unit_test /usr/local/bin/ovg_unit_test "binary" 0755 0:0
#	install_file eglinfo /usr/local/bin/eglinfo "binary" 0755 0:0



#	install_file xtest /usr/local/bin/xtest "binary" 0755 0:0

	install_file xgles1test1 /usr/local/bin/xgles1test1 "binary" 0755 0:0
	install_file xmultiegltest /usr/local/bin/xmultiegltest "binary" 0755 0:0
	install_file xgles1_texture_stream /usr/local/bin/xgles1_texture_stream "binary" 0755 0:0

	install_file xgles2test1 /usr/local/bin/xgles2test1 "binary" 0755 0:0
	install_file xgles2_texture_stream /usr/local/bin/xgles2_texture_stream "binary" 0755 0:0

	install_file xovg_unit_test /usr/local/bin/xovg_unit_test "binary" 0755 0:0
	install_file xeglinfo /usr/local/bin/eglinfo "binary" 0755 0:0
	install_file export_test /usr/local/bin/export_test "binary" 0755 0:0

}

install_X()
{
	[ -f pvr_drv.so ] &&
		install_file pvr_drv.so /usr/lib/xorg/modules/drivers/pvr_drv.so "X.Org PVR DDX module" 0755 0:0
}

# Work out if there are any special instructions.
#
while [ "$1" ]; do
	case "$1" in
	-v|--verbose)
		VERBOSE=v;
		;;
	-r|--root)
		DISCIMAGE=$2;
		shift;
		;;
	-u|--uninstall)
		UNINSTALL=y
		;;
	-n)	DOIT=echo
		;;
	--no-pvr)
		NO_PVR=y
		;;
	--no-x)
		NO_X=y
		;;
	--no-display)
		NO_DISPLAYMOD=y
		;;
	--no-bcdevice)
		NO_BCDEVICE=y
		;;
	-h | --help | *)	
		usage
		;;
	esac
	shift
done

# Find out where we are?  On the target?  On the host?
#
case `uname -m` in
arm*)	host=0;
		from=target;
		DISCIMAGE=/;
		;;
sh*)	host=0;
		from=target;
		DISCIMAGE=/;
		;;
i?86*)	host=1;
		from=host;
		if [ -z "$DISCIMAGE" ]; then	
			echo "DISCIMAGE must be set for installation to be possible." >&2
			exit 1
		fi
		;;
x86_64*)	host=1;
		from=host;
		if [ -z "$DISCIMAGE" ]; then	
			echo "DISCIMAGE must be set for installation to be possible." >&2
			exit 1
		fi
		;;
*)		echo "Don't know host to perform on machine type `uname -m`" >&2;
		exit 1
		;;
esac

if [ ! -d "$DISCIMAGE" ]; then
	echo "$0: $DISCIMAGE does not exist." >&2
	exit 1
fi

echo
echo "Installing PowerVR Consumer/Embedded DDK $PVRVERSION on $from"
echo
echo "File system installation root is $DISCIMAGE"
echo

# Uninstall whatever's there already.
#
uninstall
[ -n "$UNINSTALL" ] && exit

#  Now start installing things we want.
#
[ -z "$NO_PVR" ] && install_pvr
[ -z "$NO_X" ] && install_X

# All done...
#
echo 
echo "Installation complete!"
if [ "$host" = 0 ]; then
   echo "You may now reboot your target."
fi
echo
